package com.biasi.partyapp;

import android.app.Activity;
import android.app.ActivityOptions;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Typeface;
import android.location.Location;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.text.SpannableString;
import android.text.method.LinkMovementMethod;
import android.text.util.Linkify;
import android.transition.ChangeBounds;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.TextView;

import com.biasi.partyapp.adapters.FestaAdapter;
import com.biasi.partyapp.interfaces.OnReadyListener;
import com.biasi.partyapp.interfaces.RecyclerViewOnClickListener;
import com.biasi.partyapp.mutants.SmoothLinearLayoutManager;
import com.biasi.partyapp.models.Festa;
import com.biasi.partyapp.util.Util;
import com.biasi.partyapp.util.UtilNet;
import com.facebook.FacebookSdk;
import com.facebook.Profile;
import com.facebook.login.LoginManager;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;
import com.mikepenz.materialdrawer.Drawer;
import com.mikepenz.materialdrawer.DrawerBuilder;
import com.mikepenz.materialdrawer.accountswitcher.AccountHeader;
import com.mikepenz.materialdrawer.accountswitcher.AccountHeaderBuilder;
import com.mikepenz.materialdrawer.model.PrimaryDrawerItem;
import com.mikepenz.materialdrawer.model.ProfileDrawerItem;
import com.mikepenz.materialdrawer.model.SectionDrawerItem;
import com.mikepenz.materialdrawer.model.interfaces.IDrawerItem;


public class Principal extends AppCompatActivity implements GoogleApiClient.ConnectionCallbacks, GoogleApiClient.OnConnectionFailedListener, LocationListener {

    private FestaAdapter adapter;
    private Toolbar mToolbar;
    private GoogleApiClient mGoogleApiClient;
    private Profile user;
    private RecyclerView mRecyclerView;

    @Override
    protected void onResume() {
        super.onResume();
        mToolbar.setTitle(Util.getTitle(this));
        mToolbar.setSubtitle(Util.getSubtitle(this));
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        FacebookSdk.sdkInitialize(getApplicationContext());

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            getWindow().setSharedElementExitTransition(new ChangeBounds());
            getWindow().setSharedElementReenterTransition(new ChangeBounds());
        }
        super.onCreate(savedInstanceState);

        if (!Util.isLogged(this)) {
            finish();
            startActivity(new Intent(this, LoginScreen.class));
            return;
        }
        setContentView(R.layout.activity_principal);

        user = Profile.getCurrentProfile();

        Util.setViewMode(this, Util.TODAY);
        mToolbar = (Toolbar) findViewById(R.id.toolbar_top);
        setSupportActionBar(mToolbar);

        setupNavegationDrawer();
        callConection();
        Util.setupImageLoader(this);
        setupRecyclerView();
    }

    private void setupRecyclerView() {
        TextView mEmptyView = (TextView) findViewById(R.id.empty_view);
        mRecyclerView = (RecyclerView) findViewById(R.id.rv_list);
        mRecyclerView.setHasFixedSize(true);

        LinearLayoutManager llm = new SmoothLinearLayoutManager(this);

        llm.setOrientation(LinearLayoutManager.VERTICAL);
        mRecyclerView.setLayoutManager(llm);

        adapter = new FestaAdapter(this, mEmptyView);
        adapter.setRecyclerViewOnClickListener(new RecyclerViewOnClickListener() {
            @Override
            public void onClick(View v, int position) {
                Intent i = new Intent(Principal.this, FestaScreen.class);
                i.putExtra("festa", ""+adapter.getItemId(position));

                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    View image = v.findViewById(R.id.image1);
                    ActivityOptions ac = ActivityOptions.makeSceneTransitionAnimation(Principal.this, image, "banner");
                    startActivity(i, ac.toBundle());
                } else {
                    startActivity(i);
                }
            }
        });
        mRecyclerView.setAdapter(adapter);
        reload();
    }


    private void reload() {
        final ProgressDialog cx = ProgressDialog.show(this, null, getString(R.string.searching), true, false);
        Festa.deleteAll(Festa.class);
        OnReadyListener ready = new OnReadyListener() {
            @Override
            public void onReady(Object obj) {
                Integer result = (Integer) obj;
                adapter.update();
                cx.dismiss();
                if (result > 0) {
                    if (result == 1) {
                        Snackbar.make(findViewById(android.R.id.content), getString(R.string.found_one), Snackbar.LENGTH_SHORT).show();
                    } else {
                        Snackbar.make(findViewById(android.R.id.content), String.format(getString(R.string.found_many), result), Snackbar.LENGTH_SHORT).show();
                    }
                    mRecyclerView.smoothScrollToPosition(0);
                } else if (result == -1) {
                    Snackbar.make(findViewById(android.R.id.content), R.string.json_error, Snackbar.LENGTH_SHORT).show();
                } else if (result == -2) {
                    Snackbar.make(findViewById(android.R.id.content), R.string.request_error, Snackbar.LENGTH_SHORT).show();
                }
            }
        };
        UtilNet.getUpdates(this, ready);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == 123) {
            if (resultCode == Activity.RESULT_OK) {
                supportInvalidateOptionsMenu();
                reload(); // Atualizar a lista

            } else if (resultCode == 456) {
                // Apagar TUDO!!
                Util.getConfigs(this).edit().clear().apply();
                LoginManager.getInstance().logOut();
                finish();
                startActivity(new Intent(this, LoginScreen.class));
            }
        }
    }

    private void setupNavegationDrawer() {
        AccountHeader headerNavegationDrawer = new AccountHeaderBuilder()
                .withActivity(this)
                .withSelectionListEnabled(false)
                .withNameTypeface(Typeface.DEFAULT_BOLD)
                .withHeaderBackground(Util.getImage(this, Util.cover))
                .addProfiles(
                        new ProfileDrawerItem()
                                .withName(user.getName())
                                .withIcon(Util.getImage(this, Util.avatar))
                )
                .build();

        Drawer navegationDrawer = new DrawerBuilder(this)
                .withToolbar(mToolbar)
                .withSelectedItem(1)
                .withAccountHeader(headerNavegationDrawer)
                .withOnDrawerItemClickListener(new Drawer.OnDrawerItemClickListener() {
                    @Override
                    public boolean onItemClick(AdapterView<?> adapterView, View view, int i, long l, IDrawerItem iDrawerItem) {
                        switch (i) {
                            case 1:
                                Util.setViewMode(Principal.this, Util.TODAY);
                                mToolbar.setTitle(Util.getTitle(Principal.this));
                                reload();
                                break;
                            case 2:
                                Util.setViewMode(Principal.this, Util.TOMORROW);
                                mToolbar.setTitle(Util.getTitle(Principal.this));
                                reload();
                                break;
                            case 3:
                                Util.setViewMode(Principal.this, Util.MONTH);
                                mToolbar.setTitle(Util.getTitle(Principal.this));
                                reload();
                                break;
                            case 5:
                                startActivityForResult(new Intent(Principal.this, ConfigScreen.class), 123);
                                break;
                            case 6:
                                String msg = String.format(getString(R.string.publish_message), UtilNet.rootUrl);
                                SpannableString s = new SpannableString(msg);
                                Linkify.addLinks(s, Linkify.ALL);
                                AlertDialog ad = new AlertDialog.Builder(Principal.this)
                                        .setTitle(R.string.publish_my_event)
                                        .setIcon(R.mipmap.ic_launcher)
                                        .setMessage(s)
                                        .setNeutralButton(R.string.close, null)
                                        .create();
                                ad.show();
                                ((TextView)ad.findViewById(android.R.id.message)).setMovementMethod(LinkMovementMethod.getInstance());
                                break;
                            case 7:
                                startActivity(new Intent(Intent.ACTION_VIEW, Uri.parse("market://details?id="+ Principal.this.getPackageName())));
                                break;
                            case 8:
                                new AlertDialog.Builder(Principal.this)
                                        .setIcon(R.mipmap.ic_launcher)
                                        .setTitle(R.string.about_title)
                                        .setMessage(R.string.about_message)
                                        .setNeutralButton(R.string.close, null)
                                        .create()
                                        .show();
                                break;
                        }
                        return false;
                    }
                })
                .build();

        navegationDrawer.addItem(new SectionDrawerItem().withName("EVENTOS"));
        navegationDrawer.addItem(new PrimaryDrawerItem()
                .withName(R.string.today)
                .withTypeface(Typeface.DEFAULT_BOLD)
                .withIcon(R.drawable.ic_today));
        navegationDrawer.addItem(new PrimaryDrawerItem()
                .withName(R.string.tomorrow)
                .withTypeface(Typeface.DEFAULT_BOLD)
                .withIcon(R.drawable.ic_tomorrow));
        navegationDrawer.addItem(new PrimaryDrawerItem()
                .withName(R.string.month)
                .withTypeface(Typeface.DEFAULT_BOLD)
                .withIcon(R.drawable.ic_month));
        navegationDrawer.addItem(new SectionDrawerItem().withName("AJUSTES"));

        navegationDrawer.addItem(new PrimaryDrawerItem()
                .withName(R.string.action_settings)
                .withTypeface(Typeface.DEFAULT_BOLD)
                .withCheckable(false));
        navegationDrawer.addItem(new PrimaryDrawerItem()
                .withName(R.string.publish_my_event)
                .withTypeface(Typeface.DEFAULT_BOLD)
                .withCheckable(false));
        navegationDrawer.addItem(new PrimaryDrawerItem()
                .withName(R.string.rate_app)
                .withTypeface(Typeface.DEFAULT_BOLD)
                .withCheckable(false));
        navegationDrawer.addItem(new PrimaryDrawerItem()
                .withName(R.string.action_about)
                .withTypeface(Typeface.DEFAULT_BOLD)
                .withCheckable(false));

    }

    private synchronized void callConection() {
        mGoogleApiClient = new GoogleApiClient.Builder(this)
                .addConnectionCallbacks(this)
                .addOnConnectionFailedListener(this)
                .addApi(LocationServices.API)
                .build();
        mGoogleApiClient.connect();
        Log.i("BIASI", "callConection");
    }

    @Override
    public void onConnected(Bundle bundle) {
        Log.i("BIASI", "onConnected");
        LocationRequest mLocationRequest = new LocationRequest()
                .setInterval(2000)
                .setFastestInterval(100)
                .setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY);

        LocationServices.FusedLocationApi.requestLocationUpdates(mGoogleApiClient, mLocationRequest, this);
        Location l = LocationServices.FusedLocationApi.getLastLocation(mGoogleApiClient);
        if (l != null) Util.setCurrentLocation(this, l);
    }

    @Override
    public void onLocationChanged(Location location) {
        Util.setCurrentLocation(this, location);
        LocationServices.FusedLocationApi.removeLocationUpdates(mGoogleApiClient, this);
    }

    @Override
    public void onConnectionSuspended(int i) {
        Log.i("BIASI", "onConnectionSuspended");
    }

    @Override
    public void onConnectionFailed(ConnectionResult connectionResult) {
        Log.i("BIASI", "onConnectionFailed -> " + connectionResult.toString());
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_principal, menu);
        MenuItem item = menu.findItem(R.id.action_switch);
        item.setIcon(Util.isGPSMode(this) ? R.drawable.ic_gps_dark : R.drawable.ic_city_dark);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem menuItem) {
        if (menuItem.getItemId() == R.id.action_switch) {
            boolean newValue = !Util.isGPSMode(this);
            Util.setGPSMode(this, newValue);
            menuItem.setIcon(newValue ? R.drawable.ic_gps_dark : R.drawable.ic_city_dark);
            mToolbar.setSubtitle(Util.getSubtitle(this));
            reload();
        }
        return super.onOptionsItemSelected(menuItem);
    }
}
