package com.biasi.partyapp.util;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.StringRequest;
import com.biasi.partyapp.R;
import com.biasi.partyapp.interfaces.OnReadyListener;
import com.biasi.partyapp.models.Festa;
import com.biasi.partyapp.mutants.HttpQueue;
import com.facebook.AccessToken;
import com.facebook.GraphRequest;
import com.facebook.GraphResponse;
import com.facebook.Profile;
import com.facebook.ProfileTracker;
import com.google.android.gms.gcm.GoogleCloudMessaging;
import com.google.android.gms.iid.InstanceID;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileOutputStream;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by Gabriel de Biasi on 15/07/2015.
 */
public class UtilNet {


    //public static final String rootUrl = "http://partyapp.ddns.net";
    public static final String rootUrl = "http://party-biasi.rhcloud.com";
    //public static final String rootUrl = "http://192.168.0.102:8000";

    public static final String partyUrl = rootUrl + "/party/%s";
    public static final String registerUrl = rootUrl + "/register";
    public static final String requestUrl = rootUrl + "/request";
    public static final String counterUrl = rootUrl + "/counter";
    public static final String mediaUrl = rootUrl + "/static/media/";

    public static void getUpdates(final Context context, final OnReadyListener listener) {
        Profile.fetchProfileForCurrentAccessToken();
        final Profile user = Profile.getCurrentProfile();
        final SharedPreferences pref = Util.getConfigs(context);

        StringRequest request = new StringRequest(Request.Method.POST, UtilNet.requestUrl, new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                /* Recebe a resposta em JSON e realiza o parser para o banco. */
                Integer retorno = Util.ResponseToDatabase(response);
                if (listener != null) {
                    listener.onReady(retorno);
                }
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Integer retorno = -2;
                if (listener != null) {
                    listener.onReady(retorno);
                }
            }
        }) {
            @Override
            protected Map<String, String> getParams() {
                Map<String, String> params = new HashMap<>();
                params.put("facebook_id", user.getId());
                params.put("style", pref.getString("style", "0"));
                params.put("viewMode", "" + Util.getViewMode(context));

                if (Util.isGPSMode(context)) {
                    params.put("cidade", "-1");
                    params.put("latitude", pref.getString("latitude", "0"));
                    params.put("longitude", pref.getString("longitude", "0"));
                    params.put("distance", "" + pref.getInt("gps_range", 5));
                } else {
                    params.put("cidade", pref.getString("city", "5003702"));
                }
                return params;
            }
        };
        HttpQueue.getInstance(context).add(request);
    }

    public static void updateFacebookData(final Context context, final OnReadyListener listener) {
        final AccessToken token = AccessToken.getCurrentAccessToken();
        final ProgressDialog cx = ProgressDialog.show(context, null, context.getString(R.string.facebook_refreshing), true);

        GraphRequest request = GraphRequest.newMeRequest(
                token,
                new GraphRequest.GraphJSONObjectCallback() {
                    @Override
                    public void onCompleted(JSONObject object, GraphResponse response) {

                        new AsyncTask<JSONObject, Void, Void>() {
                            @Override
                            protected Void doInBackground(JSONObject... params) {
                                URL url;
                                try {
                                    JSONObject object = params[0];

                                    url = new URL(object.getJSONObject("cover").getString("source"));
                                    Bitmap bmp1 = BitmapFactory.decodeStream(url.openConnection().getInputStream());

                                    url = new URL(object.getJSONObject("picture").getJSONObject("data").getString("url"));
                                    Bitmap bmp2 = BitmapFactory.decodeStream(url.openConnection().getInputStream());

                                    File picture_file = new File(context.getFilesDir(), Util.avatar);
                                    File cover_file = new File(context.getFilesDir(), Util.cover);

                                    FileOutputStream fos = new FileOutputStream(cover_file);
                                    bmp1.compress(Bitmap.CompressFormat.JPEG, 100, fos);
                                    fos.close();

                                    fos = new FileOutputStream(picture_file);
                                    bmp2.compress(Bitmap.CompressFormat.JPEG, 100, fos);
                                    fos.close();


                                } catch (Exception e) {
                                    Log.e(Util.TAG, e.getMessage());
                                    Toast.makeText(context, "Algo errado aconteceu!\n" + e.toString(), Toast.LENGTH_SHORT).show();
                                }
                                return null;
                            }

                            @Override
                            protected void onPostExecute(Void params) {
                                cx.dismiss();
                                if (listener != null) {
                                    listener.onReady(null);
                                }
                            }
                        }.execute(object);
                    }
                });
        Bundle parameters = new Bundle();
        parameters.putString("fields", "picture,cover");
        request.setParameters(parameters);
        request.executeAsync();
    }

    public static void registerUser(final Context context, final OnReadyListener listener, final boolean showDialog) {
        final ProgressDialog cx;
        if(showDialog) {
            cx = ProgressDialog.show(context, null, context.getString(R.string.server_registering), true);
        } else {
            cx = null;
        }
        final StringRequest request = new StringRequest(Request.Method.POST, UtilNet.registerUrl, new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                if(showDialog) cx.dismiss();
                try {
                    JSONObject obj = new JSONObject(response);
                    if (obj.getString("status").equals("SUCCESS")) {
                        Util.getConfigs(context).edit().putBoolean("isRegistred", true).apply();
                        UtilNet.updateFacebookData(context, listener);
                    } else {
                        Toast.makeText(context, context.getString(R.string.register_error), Toast.LENGTH_SHORT).show();
                    }
                } catch (JSONException e) {
                    Toast.makeText(context, context.getString(R.string.json_error), Toast.LENGTH_SHORT).show();
                }
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                if(showDialog) cx.dismiss();
                Toast.makeText(context, context.getString(R.string.internet_error) + error.getMessage(), Toast.LENGTH_SHORT).show();
            }
        }) {
            @Override
            protected Map<String, String> getParams() {
                Profile user = Profile.getCurrentProfile();
                Map<String, String> params = new HashMap<>();
                params.put("facebook_id", user.getId());
                params.put("name", user.getName());
                params.put("gcm_id", Util.getConfigs(context).getString("gcm_id", ""));
                params.put("aparelho", "(" + Build.DEVICE + ") " + Build.MODEL + " " + Build.PRODUCT);
                return params;
            }
        };

        if (Profile.getCurrentProfile() == null) {
            ProfileTracker pf = new ProfileTracker() {
                @Override
                protected void onCurrentProfileChanged(Profile profile, Profile profile1) {
                    stopTracking();
                    Profile.setCurrentProfile(profile1);
                    HttpQueue.getInstance(context).add(request);
                }
            };
            pf.startTracking();
            Profile.fetchProfileForCurrentAccessToken();
        } else {
            HttpQueue.getInstance(context).add(request);
        }
    }

    public static void registerGCM(final Context context, final OnReadyListener callback) {
        new AsyncTask<Void, Void, String>() {

            ProgressDialog cx;

            @Override
            protected void onPreExecute() {
                cx = ProgressDialog.show(context, null, context.getString(R.string.gcm_registering), true);
            }

            @Override
            protected String doInBackground(Void... params) {
                String token = "";
                try {
                    InstanceID id = InstanceID.getInstance(context);
                    token = id.getToken(context.getString(R.string.gcm_sender_id), GoogleCloudMessaging.INSTANCE_ID_SCOPE);
                } catch (Exception e) {
                    Log.e(Util.TAG, e.getMessage());
                }
                return token;
            }

            @Override
            protected void onPostExecute(String s) {
                cx.dismiss();
                if(!s.isEmpty()) {
                    Util.getConfigs(context).edit().putString("gcm_id", s).apply();
                    UtilNet.registerUser(context, callback, true);
                } else {
                    Toast.makeText(context, context.getString(R.string.register_error), Toast.LENGTH_SHORT).show();
                }
            }
        }.execute();
    }

    public static void countRequest(Context context, final Festa f) {
        StringRequest request = new StringRequest(Request.Method.POST, UtilNet.counterUrl, new Response.Listener<String>() {
            @Override
            public void onResponse(String response) {
                if(response.equals("1")){
                    Log.i(Util.TAG, "Count request sent for ID " + f.getId());
                } else {
                    Log.i(Util.TAG, "Count request sent, but with error for ID " + f.getId());
                }
            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Log.e(Util.TAG, "Can't send count request for ID " + f.getId());
            }
        }) {
            @Override
            protected Map<String, String> getParams() {
                Profile user = Profile.getCurrentProfile();
                Map<String, String> params = new HashMap<>();
                params.put("facebook_id", user.getId());
                params.put("festa_id", String.valueOf(f.getId()));
                return params;
            }
        };
        HttpQueue.getInstance(context).add(request);
    }
}
