from django.contrib.gis.db import models
from django.contrib.auth.models import User
import uuid
from django.contrib.gis.geos import Point
from location_field.models.spatial import LocationField

# Helper para upload.
def fileHelper(instance, filename):
	ext = filename.split('.')[-1]
	result = 'banners/{}.{}'
	return result.format(uuid.uuid1(), ext)

class Profile(models.Model):
	nome = models.CharField(max_length=100)
	facebook = models.CharField(max_length=50)
	data = models.DateTimeField(auto_now_add=True)
	reqs = models.IntegerField(default=0)
	gcm = models.CharField(max_length=200)
	aparelho = models.CharField(max_length=100)

	cidade = models.IntegerField()
	location = LocationField(based_fields=[cidade], zoom=12, default=Point(1, 1))
	objects = models.GeoManager()

	def __unicode__(self):
		return "[Profile=" + self.nome + "]"

class Promoter(models.Model):
	user = models.OneToOneField(User)
	cidade = models.IntegerField()
	telefone = models.CharField(max_length=30)
	identidade = models.CharField(max_length=30)
	max_eventos = models.IntegerField(default=3)

	def __unicode__(self):
		return "[Promoter=" + self.user.first_name + "]"

class Festa(models.Model):
	banner = models.ImageField(upload_to=fileHelper, help_text='Utilize uma imagem com aspecto 16:9.')
	titulo = models.CharField(max_length=40)
	descricao = models.TextField(max_length=1000)
	promoter = models.ForeignKey(User)
	profiles = models.ManyToManyField(Profile)
	data = models.DateTimeField()
	site = models.URLField(blank=True)
	style = models.IntegerField()

	cidade = models.IntegerField()
	location = LocationField(based_fields=[cidade], zoom=12, default=Point(1, 1))

	objects = models.GeoManager()

	def __unicode__(self):
		return "Festa: ["+ self.titulo + "]"

	class Meta:
		ordering = ['data']


