# -*- coding: utf-8 -*-

from django.shortcuts import render, get_object_or_404
from django.db.models import Q
from django.core.mail import send_mail
from Party.models import *
from django.core.urlresolvers import reverse
from Party.forms import *
from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseRedirect
from django.contrib import auth
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required
from django.core import serializers
from django.core.mail import send_mail
from django.contrib.gis.geos import Point
from django.conf import settings
from django.contrib.gis.measure import D
from django.utils import timezone
from datetime import datetime, timedelta, time
import json

# Página Principal
def principal(request):
	login_form = LoginForm(request.POST or None)
	if request.method == 'POST':
		if login_form.is_valid():
			user = login_form.login()
			if user:
				auth.login(request, user)
				return HttpResponseRedirect(reverse('dashboard'))
	return render(request, 'index.html', {
		'login_form': login_form,
		})

@login_required
def dashboard(request):
	festas = Festa.objects.filter(promoter=request.user)
	return render(request, 'dashboard.html', {
		'festas': festas
		})

@login_required
def new_event(request, festa_id):
	# Controle de new/edit
	if festa_id is not None:
		op = get_object_or_404(Festa, pk=festa_id)
		if op.promoter != request.user:
			return HttpResponseRedirect(reverse('dashboard'))
		data = None
		city = {'cidade': op.cidade}
	else:
		city = {'cidade': request.user.promoter.cidade}
		op = None
		data = {'data': timezone.now()}

	# Controle de quantidade de eventos por promoter
	if request.user.promoter.max_eventos <= request.user.festa_set.count() and op is None:
		return HttpResponseRedirect(reverse('dashboard'))

	# Método POST, tenta instanciar e salvar
	if request.method == 'POST':
		festa_form = FestaForm(request.POST, request.FILES, instance=op)
		cidade_form = CidadeForm(request.POST)
		if festa_form.is_valid() and cidade_form.is_valid():
			festa = festa_form.save(commit=False)
			festa.promoter = request.user
			festa.cidade = cidade_form.cleaned_data['cidade']
			festa.save()
			return HttpResponseRedirect(reverse('dashboard'))
	else:
		festa_form = FestaForm(instance=op, initial=data)
		cidade_form = CidadeForm(initial=city)
	return render(request, 'new-event.html', {
		'festa_form': festa_form,
		'new': op is None,
		'cidade_form': cidade_form
		})

def delete_event(request, festa_id):
	f = get_object_or_404(Festa, pk=festa_id)
	if f.promoter == request.user:
		f.delete()
	return HttpResponseRedirect(reverse('dashboard'))

# Request de Festa
@csrf_exempt
def party_request(request, party_id):
	if request.method == 'POST':
		festa = Festa.objects.filter(pk=int(party_id))
		if len(festa) > 0:
			return HttpResponse(serializers.serialize("json", festa), content_type='application/json')	
	return HttpResponseBadRequest()

# Request do App
@csrf_exempt
def app_count_request(request):
	user_id = request.POST.get('facebook_id', '-1')
	festa_id = request.POST.get('festa_id', '-1')
	user = get_object_or_404(Profile, facebook=user_id)
	festa = get_object_or_404(Festa, pk=festa_id)
	festa.profiles.add(user)
	return HttpResponse("1")

# Request do App
@csrf_exempt
def app_request(request):
	if request.method == 'POST':
		user_id = request.POST.get('facebook_id', '-1')
		user = get_object_or_404(Profile, facebook=user_id)
		
		# Incrementa o contador de requisições deste usuário.
		user.reqs += 1
		
		# Verifica se a busca será filtrada por estilo da festa.
		style = int(request.POST.get('style', '-1'))
		if style > 0:
			style_Q = Q(style=style)
		elif style == 0:
			style_Q = Q()
		else:
			return HttpResponseBadRequest()

		# Processa o intervalo de datas
		mode = int(request.POST.get('viewMode', '-1'))
		now = timezone.now()
		# HOJE
		# Mostra todos os eventos até às 23:59:59 do dia atual.
		if mode == 0:
			futuro = datetime.combine(now+timedelta(days=1), time(0,0))
			data_Q = Q(data__range=[now, futuro])

		# PRÓXIMOS DIAS
		# Mostra os eventos para 7 dias.
		elif mode == 1:
			data_Q = Q(data__range=[now, now+timedelta(days=7)])

		# MÊS INTEIRO
		# Mostra eventos do mês.
		elif mode == 2:
			data_Q = Q(data__range=[now, now+timedelta(days=30)])

		# MODO CHEIO (sem filtro de data)
		# Não utilizado no aplicativo, mas pelo menos está implementado.
		elif mode == 3:
			data_Q = Q()

		# SEM MODO
		# Retorna Bad Request
		else:
			return HttpResponseBadRequest()

		cidade = request.POST.get('cidade', '-1')
		# Se a cidade não for especificada, é utilizado os dados de GPS.
		if cidade == '-1':
			latitude = float(request.POST.get('latitude', '0'))
			longitude = float(request.POST.get('longitude', '0'))
			distance = int(request.POST.get('distance', '0'))
			point = Point(longitude, latitude)
			user.location = point
			location_Q = Q(location__distance_lte=(point, D(km=distance)))
		else:
			user.cidade = cidade
			location_Q = Q(cidade=cidade)
			
		resultado = Festa.objects.filter(data_Q & style_Q & location_Q)
		user.save()
		return HttpResponse(serializers.serialize("json", resultado), content_type='application/json')
	
	# Se algo der errado, retorna BadRequest.
	return HttpResponseBadRequest()


# Register do App
@csrf_exempt
def app_register(request):
	if request.method == 'POST':
		facebook_id = request.POST.get('facebook_id', '')
		nome = request.POST.get('name', '')
		aparelho = request.POST.get('aparelho', '')
		gcm = request.POST.get('gcm_id', '')
		perfil, novo = Profile.objects.get_or_create(facebook=facebook_id, defaults={
			'aparelho': aparelho,
			'gcm': gcm,
			'cidade': 5002704,
			'nome': nome,
		})
		retorno = {'status': 'SUCCESS' if perfil else 'FAIL'}
		return HttpResponse(json.dumps(retorno), content_type='application/json')
	else:
		return HttpResponseBadRequest()

"""
def register(request):
	if request.method == 'POST':
		register_form = RegisterForm(request.POST)
		promoter_form = PromoterForm(request.POST)
		if register_form.is_valid() and promoter_form.is_valid():
			if register_form.cleaned_data['password1'] == register_form.cleaned_data['password2']:
				user = register_form.save(commit=False)
				user.set_password(register_form.cleaned_data['password1'])
				user.save()
				promoter = promoter_form.save(commit=False)
				promoter.user = user
				promoter.save()
			
		return HttpResponseRedirect(reverse('principal'))
	else:
		raise Http404
"""

def send_email(request):
	if request.method == 'POST':
		nome = request.POST.get('name', '')
		email = request.POST.get('email', '')
		conteudo = request.POST.get('conteudo', '')
		if nome != '' and email != '' and conteudo != '':
			send_mail(u'Novo contato no Party', u'Nome: {}\nE-mail: {}\nData: {}\n\n== Inicio da Mensagem == \n{}\n== Fim da Mensagem ==\n'.format(nome, email, timezone.now(), conteudo), settings.EMAIL_HOST_USER,[settings.EMAIL_SEND_TO], fail_silently=False)
		return HttpResponseRedirect(reverse('principal'))
	raise Http404

def logout(request):
	auth.logout(request)
	return HttpResponseRedirect(reverse('principal'))
